]> git.sesse.net Git - movit/blobdiff - effect.h
Change so that all modifications to the graph (meta-effects, colorspace information...
[movit] / effect.h
index fcb752638115dd6297bccb918cd606dd3b535645..ef58392715ea7ade816bfc440c1eec65bd3ee410 100644 (file)
--- a/effect.h
+++ b/effect.h
 #include <string>
 #include <vector>
 
-#include <GL/gl.h>
+#include <assert.h>
+
+#include "opengl.h"
 
 class EffectChain;
+class Node;
 
 // Can alias on a float[2].
 struct Point2D {
@@ -44,7 +47,12 @@ void set_uniform_vec3(GLuint glsl_program_num, const std::string &prefix, const
 void set_uniform_vec4_array(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values, size_t num_values);
 
 class Effect {
-public: 
+public:
+       // An identifier for this type of effect, mostly used for debug output
+       // (but some special names, like "ColorSpaceConversionEffect", holds special
+       // meaning). Same as the class name is fine.
+       virtual std::string effect_type_id() const = 0;
+
        // Whether this effects expects its input (and output) to be in
        // linear gamma, ie. without an applied gamma curve. Most effects
        // will want this, although the ones that never actually look at
@@ -60,7 +68,9 @@ public:
        // set of RGB primaries; you would currently not get YCbCr
        // or something similar).
        //
-       // Again, most effects will want this.
+       // Again, most effects will want this, but you can set it to false
+       // if you process each channel independently, equally _and_
+       // in a linear fashion.
        virtual bool needs_srgb_primaries() const { return true; }
 
        // Whether this effect expects its input to come directly from
@@ -94,16 +104,39 @@ public:
        // needs mipmaps, you will also get them).
        virtual bool needs_mipmaps() const { return false; }
 
+       // Whether this effect wants to output to a different size than
+       // its input(s). If you set this to true, the output will be
+       // bounced to a texture (similarly to if the next effect set
+       // needs_texture_bounce()).
+       virtual bool changes_output_size() const { return false; }
+
+       // If changes_output_size() is true, you must implement this to tell
+       // the framework what output size you want.
+       //
+       // Note that it is explicitly allowed to change width and height
+       // from frame to frame; EffectChain will reallocate textures as needed.
+       virtual void get_output_size(unsigned *width, unsigned *height) const {
+               assert(false);
+       }
+
        // How many inputs this effect will take (a fixed number).
        // If you have only one input, it will be called INPUT() in GLSL;
        // if you have several, they will be INPUT1(), INPUT2(), and so on.
        virtual unsigned num_inputs() const { return 1; }
 
-       // Requests that this effect adds itself to the given effect chain.
-       // For most effects, the default will be fine, but for effects that
-       // consist of multiple passes, it is often useful to replace this
-       // with something that adds completely different things to the chain.
-       virtual void add_self_to_effect_chain(EffectChain *graph, const std::vector<Effect *> &inputs);
+       // Let the effect rewrite the effect chain as it sees fit.
+       // Most effects won't need to do this, but this is very useful
+       // if you have an effect that consists of multiple sub-effects
+       // (for instance, two passes). The effect is given to its own
+       // pointer, and it can add new ones (by using add_node()
+       // and connect_node()) as it sees fit. This is called at
+       // EffectChain::finalize() time, when the entire graph is known,
+       // in the order that the effects were originally added.
+       //
+       // Note that if the effect wants to take itself entirely out
+       // of the chain, it must set “disabled” to true and then disconnect
+       // itself from all other effects.
+       virtual void rewrite_graph(EffectChain *graph, Node *self) {}
 
        // Outputs one GLSL uniform declaration for each registered parameter
        // (see below), with the right prefix prepended to each uniform name.
@@ -113,17 +146,19 @@ public:
        // Returns the GLSL fragment shader string for this effect.
        virtual std::string output_fragment_shader() = 0;
 
// Set all uniforms the shader needs in the current GL context.
// The default implementation sets one uniform per registered parameter.
        //
        // <sampler_num> is the first free texture sampler. If you want to use
        // textures, you can bind a texture to GL_TEXTURE0 + <sampler_num>,
        // and then increment the number (so that the next effect in the chain
        // will use a different sampler).
-       //
-       // NOTE: Currently this is also abused a bit to set other GL state
-       // the effect might need.
-       virtual void set_uniforms(GLuint glsl_program_num, const std::string& prefix, unsigned *sampler_num);
+       virtual void set_gl_state(GLuint glsl_program_num, const std::string& prefix, unsigned *sampler_num);
+
+       // If you set any special OpenGL state in set_gl_state(), you can clear it
+       // after rendering here. The default implementation does nothing.
+       virtual void clear_gl_state();
 
        // Set a parameter; intended to be called from user code.
        // Neither of these take ownership of the pointer.
@@ -113,17 +146,19 @@ public:
        virtual std::string output_fragment_shader() = 0;
 
-       // Set all uniforms the shader needs in the current GL context.
-       // The default implementation sets one uniform per registered parameter.
+       // Set all OpenGL state that this effect needs before rendering.
+       // The default implementation sets one uniform per registered parameter,
+       // but no other state.
        //
        // <sampler_num> is the first free texture sampler. If you want to use